      SUBROUTINE COLNORM(Mol,Temp)
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2008 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c 
c
c      REVISIONS
c
c      06/2008  Revised algorithm to minimize instabilities and improve 
c               convergence; Still some instability related to absent states
c               at low energies.
c
c      04/2006  General overhaul to improve accuracy, especially at 
c               low energies.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c 
c
c	Normalization for a down-step model given
c		in FUNCTION PDOWN(E,EE,Mol,Temp,Hs)
c
c		Mol	= index of molecule
c		Temp	= translational temperature
c	 	E	= initial energy
c		EE	= final energy
c		Hs	= characteristic stepsize at energy E (calculated by PDOWN)
c
c	Note that COLLUP and CNORM are 'double arrays' with the first
c	imax1+1 elements starting at E = 0 with Egrain1 spacing; the remaining
c	elements are at Egrain2 spacing, starting at E = 0.
c
c	Coefficients stored in array DC
c	Probability of up-steps stored in COLLUP(MaxWell,Isize)
c	Normalization factor (up + down) stored in CNORM(MaxWell,Isize)
c
c       ERRo    = Convergence criterion declared in Declare3.inc include file
c	EDIFF 	= Minimum integration interval (cm-1)
c	FRACT  	= Step-size relative to 'Hs'
c	ITERS  	= Iterations on up-steps
c
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare3.inc'
 
      DOUBLE PRECISION PDOWN , DENVR , Temp , CNORMI , CNORMJ , 
     &                 RATIO , DELMIN , RATT , RATT0 , DUP

 
      DOUBLE PRECISION B , DENI , E , EE , Hs , PROB , H , DENJ , TERM , 
     &                 SPAN , TLAST , TEST , DOUBNTERP , a , BL ,
     &                 AA , TESTH , CC , Elow , arg , Ebare
 
      INTEGER I , J , kk , Mol , Igg , Ilow , jc ,
     &        IE , Istart

      DOUBLE PRECISION CN , PUP , PUPmax , denslo , Etherm
      DIMENSION CN(Imax) , denslo(Imax)
 
      EXTERNAL PDOWN , DENVR , RAN1 , DOUBNTERP , Etherm
 
      SAVE 

      Statest = 0.5d+00/Egrain1      ! Criterion for whether a state is present within an energy grain

      WRITE (*,99001) Mol
 
c	Integrate over UP steps (temporarily store up-step integrals in COLLUP)
c
       DO 100 I = 1 , Isize
 
         IF ( I.LE.imax1 ) THEN
            E = (I-1)*Egrain1
           ELSE
            E = (I-imax1-1)*Egrain2
         ENDIF

         DENI = DENVR(Mol,E)                                 ! Density at E(I)
         IF ( I. EQ. 1 ) DENI = 1.0                          ! Special case for E=0

         IF ( E.GE.0.0d+00 .AND. DENI.GE.Statest ) THEN      !  ONLY IF STATE, OTHERWISE SKIP
            TEST = 1.0
            COLLUP(Mol,I) = 0.0D+00     ! Temporary storage of UP-normalization
            EE = E
            PROB = PDOWN(E,EE,Mol,Temp,Hs)
            TLAST = 1.0d+00                                     ! Initial Hs and 'old' TERM
            DELMIN = Viblo(Mol) + 0.5*Hs*(Temp/hor)/(Hs + (Temp/hor))

            DO WHILE ( TEST.GT.ERRo )                            ! ********** UP STEPS **********
               IF ( EE.GT.Emax1 ) THEN
                  H = FRACT*Hs*(Temp/hor)/(Hs + (Temp/hor))    
                ELSE
                  H = Egrain1
               ENDIF

               EE = EE + H                                     ! UP-steps

               DENJ = DENVR(Mol,EE)
               IF ( DENJ .GE. Statest ) THEN
                  RATIO = DENJ/DENI
                  B = EXP(-(EE-E)*hor/Temp)                     ! EE > E
                  PROB = PDOWN(EE,E,Mol,Temp,Hs)               
                  TERM = PROB*B*RATIO
               ELSE
                  TERM = 0.0D+00
               ENDIF
 
               IF ( EE .GT. Emax1 ) THEN
                  a = -LOG(TERM/TLAST)/H                        ! Exponential form
                  AA = TLAST                                    ! Exponential form
                  SPAN = (AA/a)*(1.0D+00-EXP(-a*H))             ! Exponential form: integral
               ELSE
                  SPAN = 0.5D+00*H*( TERM + TLAST )             ! Trapezoidal Rule: integral
               ENDIF
 
               COLLUP(Mol,I) = COLLUP(Mol,I) + SPAN
 
               IF ( SPAN.GT.0.0 .AND. ABS(E-EE).GT.DELMIN )
     &              TEST = ABS(SPAN/COLLUP(Mol,I))
               TLAST = TERM                                     ! 'old' TERM
            ENDDO
         ENDIF
 100  CONTINUE

      WRITE (*,99002)
 
c		! ********** DOWN STEPS ***********
c*************************************************************
c 
      Ilow = Emax1/Egrain2 + 2 + imax1                 ! index of lowest element in upper half not overlapped with lower half

      CNORM(Mol,1) = COLLUP(Mol,1)
      CN(1) = 0.0d+00
 
        DO 250 I = 2 , Isize

        IF ( I.LE.imax1 ) THEN
           E = (I-1)*Egrain1
          ELSE
           E = (I-imax1-1)*Egrain2
        ENDIF

        DENI = EXP( Dens(Mol,I) )                   ! Density at E(I)

	IF ( DENI.LT.Statest ) THEN
          CNORM(Mol,I) = 0.0d+00
	ELSE                                        ! PROCEED ONLY IF STATE PRESENT

            EE = E
            PROB = PDOWN(E,E,Mol,Temp,Hs)                    ! to get Hs
            DELMIN = Viblo(Mol) + 0.5*Hs
            TLAST = 1.0d+00                                  ! Initial Hs and 'old' TERM
            TEST = 1.0
            CN(I) = 0.0d+00

            DO WHILE ( TEST.GT.ERRo .AND. EE .GE. Egrain1 )
              IF ( EE.GT.Emax1 ) THEN
                H = FRACT*Hs
                H = MIN( H , EE-Emax1 )
               ELSE
                H = Egrain1
              ENDIF
              IF ( H.GT.EE ) H = EE

              EE = EE - H                                  ! DOWN-steps
              DENJ = DENVR(Mol,EE)
              IF ( DENJ.GE.Statest ) THEN
                PROB = PDOWN(E,EE,Mol,Temp,Hs)                  
                TERM = PROB
               ELSE
                TERM = 0.0D+00
              ENDIF
 
              IF ( EE-H .GE. Emax1 ) THEN
                a = -LOG(TERM/TLAST)/H                     ! Exponential form
                AA = TLAST                                 ! Exponential form
                SPAN = (AA/a)*(1.0D+00-EXP(-a*H))          ! Exponential form
               ELSE
                SPAN = 0.5D+00*H*( TERM + TLAST )          ! Trapezoidal Rule
              ENDIF
              TLAST = TERM                          ! 'old' value of TERM
              CN(I) = CN(I) + SPAN
 
              IF ( SPAN.GT.0.0 .AND. ABS(E-EE).GT.DELMIN ) 
     &             TEST = ABS(SPAN/CN(I))

            ENDDO

            IF ( ( COLLUP(Mol,I)+CN(I) ) .GT. 0.0 ) THEN
              CNORM(Mol,I) = ( COLLUP(Mol,I) + CN(I) )         
             ELSE
              CNORM(Mol,I) = 0.0d+00
            ENDIF

        ENDIF           ! End test for state present
 
 250  CONTINUE          ! end energy steps

      CNORM(Mol,1) = COLLUP(Mol,1) 
      CNORM(Mol,imax1+1) = CNORM(Mol,1)
 
c	Now calculate final COLLUP array

      DO 500 I = 1 , Isize
        IF ( CNORM(Mol,I) .GT. 0.d+00 ) THEN
          COLLUP(Mol,I) = COLLUP(Mol,I)/CNORM(Mol,I)             ! Evaluate up-probability array
         ELSE
          COLLUP(Mol,I) = 0.0d+00
        ENDIF
 500  CONTINUE

c      Now use values for lower half of double array to replace overlapped values in upper half

      DO I = imax1+2 , Ilow-1
         EE = (I-imax1-1)*Egrain2
         CNORM(Mol,I)  = DOUBNTERP(EE,Mol,CNORM)
         COLLUP(Mol,I) = DOUBNTERP(EE,Mol,COLLUP)
      END DO

      COLLUP(Mol,1) = 1.0d+00
      COLLUP(Mol,imax1+1) = COLLUP(Mol,1)
      CNORM(Mol,imax1+1) = CNORM(Mol,1)

      WRITE (*,99003)
 
      RETURN
 
99001 FORMAT ('Entering Subroutine COLNORM: Molecule #',I2)
 
99002 FORMAT ('    Finished up steps, starting down-steps')
 
99003 FORMAT ('Leaving Subroutine COLNORM'/)
      END
