c	Declarations
c
c       UNIT CONVERSIONS
c
c      NIST Physical Constants (1998, 2000)
c
c       jtocm   = kJ/mol to cm-1
c       caltoj  = kcal/mol to kJ/mol  (thermochemical calorie)
c       caltocm = kcal/mol to cm-1    (thermochemical calorie)
c
      DOUBLE PRECISION clight , hor , jtocm , jtocal , caltocm
      DOUBLE PRECISION plank , avogadro
      PARAMETER ( clight=2.99792458d+10 ,
     &            hor=1.4387752d+00 ,
     &            jtocm=83.59347d+00 , 
     &            jtocal=4.184d+00 , 
     &            caltocm=349.7551d+00 ,
     &            plank=6.62606896d-34 ,
     &            avogadro=6.02214179d+23)
c
c	Isize	= size of double array SELECTED BY USER
c
c       Imax    = upper limit size of double array
c	MaxWell	= maximum number of wells
c	MaxProd = maximum of product sets
c	MaxChan	= maximum number of reaction channels from any given well
c	MaxRxn	= maximum number of reactions, summed over all wells
c       Jsize   = size of the cumulative initial energy distribution
c
      INTEGER Isize , Jsize
      COMMON/SIZE/ Isize , Jsize
      INTEGER Imax , MaxWell , MaxProd , MaxChan , MaxRxn
      PARAMETER (Imax=14000,MaxWell=50,MaxProd=50,MaxChan=25,MaxRxn=100)
c
c
c
c	Input/output unit numbers, etc.
c
      INTEGER KIN , KOUT , KTXT , KARY , KDIS , KRAT , KSTD , KFLX ,
     &               KSMM
      COMMON /UNIT  / KIN , KOUT , KTXT , KARY , KDIS , KRAT , KSTD , 
     &                KFLX , KSMM
      CHARACTER cut*46, dumcut*46
      PARAMETER (cut='**************INPUT DATA SUMMARY**************')
c
c
c	Energy grains and maximum energies for double arrays (cm-1)
c
      DOUBLE PRECISION Egrain1 , Egrain2 , Emax1 , Emax2
      INTEGER imax1
      COMMON /ENERGY/ Egrain1 , Emax1 , Emax2 , Egrain2 , imax1
c
c
c	NWells	= number of wells in a given calculation
c	NProds	= number of product sets in a given calculation
c	IMol	= index number of any given molecule
c	Molsym	= symmetry number of a given molecule
c	Molele	= electronic partition function of a given molecule
c	Molopt	= number of optical isomers of a given molecule
c	HMol	= enthalpy (actually internal energy) of a molecule or product set
c	MolMom	= moment of inertia for 2-D external rotation of a given molceule
c	MolName	= name of a given molecule
c	Viblo	= lowest vibrational frequency in Mol
c
      INTEGER NWells , NProds , IMol , Molsym , Molopt
      DOUBLE PRECISION HMol , MolMom , Viblo , Molele 
      CHARACTER MolName*10 , LJQM*2 , LJ*2 , QM*2
      PARAMETER (LJ='LJ',QM='QM')
      COMMON /MOL   / HMol(MaxWell+MaxProd) , MolMom(MaxWell) , 
     &                 Viblo(MaxWell) , IMol(MaxWell+MaxProd) , 
     &                 Molele(MaxWell) , Molsym(MaxWell), 
     &                 Molopt(MaxWell) ,  NWells , NProds
      COMMON /MOLC  / MolName(MaxWell+MaxProd) , LJQM(MaxWell)
c
c
c	Jto	= index number of product molecule
c	Jrev	= flag for reversible reaction
c	Nchan	= number of rxn channels from a given molecule
c	TSsym	= symmetry number of a transiton state
c	TSele	= electronic partition function of a transition state
c	TSopt	= number of optical isomers of a transition state
c	Jread	= flag for reading sums, or k(E)s from an external file
c	Afac	= A-factor used with Inverse Laplace Transform method
c	Eo	= critical energy for a reaction channel
c         Eor       = critical energy for a reaction channel, corrected for rotational energy
c	Hts	= enthalpy (internal energy) of a transition state
c         NCENT     = flag for applying quasi-diatomic centrifugal correction
c	TSmom	= moment of inertia for 2-D external rotation of a transition state
c	TSname	= name of a transition state
c
      INTEGER Jto , Jrev , Nchan , TSsym , TSopt , Jread , NCENT
      DOUBLE PRECISION Afac , Eo , Eor , Hts , TSmom, V1, Press, TSele
      CHARACTER*10 TSname
      COMMON /RXN   / Afac(MaxWell,MaxChan) , Eo(MaxWell,MaxChan) , 
     &                Eor(MaxWell,MaxChan) , TSele(MaxWell,MaxChan) , 
     &                Hts(MaxWell,MaxChan) , TSmom(MaxWell,MaxChan) , 
     &                Jto(MaxWell,MaxChan) , Nchan(MaxWell) , 
     &                Jrev(MaxWell,MaxChan) , TSsym(MaxWell,MaxChan) , 
     &                TSopt(MaxWell,MaxChan) , 
     &                Jread(MaxWell,MaxChan) , NCENT(MaxWell,MaxChan) , 
     &                V1(MaxWell,MaxChan),
     &                Press(MaxWell,MaxChan)
      COMMON /RXNC  / TSname(MaxWell,MaxChan)
c
c
c	CollK	  = bimolecular collision frequency rate constant
c       CNORMref  = reference normalization for collision frequency at rxn Eor ----> NEW UP-STEP COLLISION METHOD
c       Eref      = reference energy for CNORMref                              ----> NEW UP-STEP COLLISION METHOD
c       QCollK    = quantum total bimolecular collision frequency rate constant
c	Dens	  = density of states for molecule Mol
c	Rate	  = k(E) for a given molecule and channel (no tunneling)
c       TRate     = k(E) with tunneling correction
c       KRate     = k(E) book keeping array containing Rate and TRate; used for output purposes only
c       DensTS    = density of states for a specific transition state; the first part of the
c		    DensTS array must be of size MaxWell*MaxChan in order to account for the case
c		    that tunneling corrections are applied to all channels from all wells
c       tunthresh = minimum tunneling probability; if the tunneling probability is less
c		    than tunthresh, calculation of the tunneling corrected sum of states
c		    for the transition state is stopped
c	Qthresh   = when the relative contribution to the partition function
c		    becomes less than Qthresh we stop evaluating the partition function
      DOUBLE PRECISION  CollK , Dens , Rate , Path , QCollK , TRate
      DOUBLE PRECISION  DensTS , TSsumm , tunthresh , Qthresh, KRate
      DOUBLE PRECISION  TSdensBK , CNORMref , Eref
      INTEGER iset,iset2
      PARAMETER(tunthresh=1.0d-12,Qthresh=1.0d-12)
      COMMON /RATE  / CollK(MaxWell) , CNORMref(MaxWell) ,
     &                Eref(MaxWell) , Dens(MaxWell,Imax) , 
     &                Rate(MaxWell,MaxChan,Imax) , 
     &                Path(MaxWell,MaxChan) , QCollK(MaxWell),
     &                TRate(MaxWell,MaxChan,Imax*2) ,
     &                KRate(MaxWell,MaxChan,Imax*2) ,
     &                DensTS(MaxWell*MaxChan,Imax)  ,
     &                TSdensBK(MaxWell,Imax*2), iset(MaxWell),
     &                iset2(MaxWell)  
c
c
c	DC	= coefficients defining a collision model
c	COLLUP	= probability of an up-step
c	CNORM	= normalization factor for collision model
c	IDUM	= random number seed
c	ITYPE	= flag designating collision model type
c
      DOUBLE PRECISION vimag
      INTEGER itun
      COMMON /TUN  / vimag(MaxWell,MaxChan), itun(MaxWell,MaxChan)
c
      DOUBLE PRECISION DC , COLLUP , CNORM , CE
      INTEGER IDUM , ITYPE
      CHARACTER LI*100
      COMMON /COLL  / COLLUP(MaxWell,Imax) , CNORM(MaxWell,Imax) , 
     &                DC(MaxWell,8) , ITYPE(MaxWell) , IDUM
      COMMON /COLLC / LI(MaxWell,5)
c
c
c	Pstart	= Normalized cumulative distrib. of starting energies
c	Hstart	= Energy step size for initial energy distribution Pstart
c	Nstart	= actual starting energies selected via random number
c
      DOUBLE PRECISION Pstart , Hstart , Edels
      INTEGER Nstart
      COMMON /START / Pstart(Imax) , Nstart(Imax) , Hstart , Edels
c
c
c	Rab(i) are the unimolecular reaction rates and collision
c	frequency calculated in STEPPER and used in BOOK1
c
      DOUBLE PRECISION Rab
      COMMON /BOSTEP/ Rab(MaxChan+1)
