      SUBROUTINE QKinftun(Mol,nchann,tx,Kinf,Ainf,Einf)
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2002 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
c
c     01/02  v1.2.0  -New Feature: can optionally choose to apply centrifugal
c                     corrections for any choice of model. Thus can apply them
c                     even when using Inverse Laplace Transform form of k(E).
c                     NOTE: this new feature affects the data file format.
c     11/01  v1.1.4  -Bug Fix: previous version applied centrifugal correction 
c                     inconsistently when calculating Arrhenius parameters 
c                     (k(inf) was correct, however).
c     09/01  v1.1.2  -Revised centrigfugal corrections to k(E)'s.
c                     See Robinson & Holbrook (1972) eq. 4.30 and 4.31.
c
c***********************************************************************

      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER Mol , nchann , i , IE , Itop , mm, i2
      DOUBLE PRECISION Q , RK , TT , E , D , DENVR , Kinf , Ainf , 
     &                 Einf , K , tx , Et , arg , Y1 , Y2 , Y3 , Y , 
     &                 X , Estep, RR, QQ , Qlast , Rlast , TTSTEP , 
     &                 TS , EC, E2, Qcheck, Qts, QQts, Qtslast
      DIMENSION K(3), TS(3)
      SAVE 
c
c	Calculates high pressure limiting unimolecular rate constant according to
c         Equation (4.30) of Robinson & Holbrook, "Unimolecular Reactions"
c         (Wiley-Interscience, London, 1972), p. 90.
c
c	Mol	= index of molecule
c	nchann	= index of rxn channel
c	t	= translational temperature
c	Kinf	= infinite pressure rate constant [Robinson & Holbrook Eq. 4.30]
c	Ainf	= A-factor for kinf
c	Einf	= Activation energy for kinf
c
c
c     Centrifugal correction at temperature = tx
c
      EC =  Eor(Mol,nchann)                 ! Includes centrifugal correction

      TTSTEP = 0.001d+00/tx                  ! Step size in 1/Temp
      TT = 1.0d+00/tx-2.d+00*TTSTEP          ! 1/Temp

      DO 1000 mm = 1,3
        K(mm) = 0.0d+00
        RK = 0.0d+00
        Q  = 0.0d+00
        RR = 0.0d+00
        QQ = 0.0d+00
        Qlast = 0.0d+00
        Rlast = 0.0d+00
        QQts=0.0d00
        Qtslast=0.0d0
        Qts=0.0d0

        TT = TT + TTSTEP                     ! 1/T
        TS(mm) = TT                          ! 1/T

        Estep = Egrain1
        E = -Estep*(imax1+1)        

      DO 100 i = 1 , 2*imax1                       ! start at imax1*Egrain1 below the reaction threshold
         E = E + Estep                             ! summation continues till end of first part of double array

        if(i.gt.imax1)then                         ! allows Q(Trans) to be calculated from 0-->inf
          QQ=exp(Dens(Mol,i-imax1)-(E*hor*TT))     ! molecule partition function;correct E and Dens
                                                   ! both start at 0 Energy and index=0 i.e. (i-imax1)=0
          Q  = Q  + 0.5*Estep*(QQ + Qlast)         ! Trapezoidal Rule
          Qlast = QQ
          QQts=D*exp(-E*hor*TT)
          Qts=Qts+0.5*Estep*(QQts+Qtslast)
          Qtslast=QQts
        else
          continue
        end if

         arg = E + EC                              ! arg is E + barrier height; starts at -Egrain1*imax1
                                                   ! below the barrier

        if(arg.ge.0)then
	    
          D = DENVR(Mol,arg)                        ! density of states of molecule evaluated at
                                                    ! the (top of the barrier - Egrain1*imax1)



	
          RR = D*exp(TRate(Mol,nchann,i)-(E*hor*TT))! this corresponds to Eq. 12 in MW pub. and is
                                                   ! the integral from (Eo-(Egrain1*imax1)) to inf
          RK = RK + 0.5*Estep*(RR + Rlast)
          Rlast = RR
        end if


 100  CONTINUE
c
c          NOW CONTINUE TO HIGHER ENERGIES
c 
      Estep = 10.*Egrain1                          ! Step size (cm-1)
      Itop = ( 25.0/(hor*TT))/Estep                ! Integral to be extended to 25 kT
      E2=E
      DO i = 1 , Itop                              ! Continuing the integral to higher energies
         E = E + Estep                             ! last energy from above loop + new Estep is 1st energy evaluated
         D = DENVR(Mol,E)
         QQ = D*exp(-E*hor*TT)
         Q  = Q  + 0.5*Estep*(QQ + Qlast)           ! Trapezoidal Rule
         Qlast = QQ
         Qcheck = dabs(QQ/Q)
         if(Qcheck.lt.Qthresh)then
           goto 17
         else
           continue
         end if

         if(E.gt.Emax2)then
           write(*,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           write(KOUT,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           STOP
          else
            continue
          end if
        end do

 17     continue

        arg=E2+EC
        do i2=1,Itop
           arg=arg+Estep
           E2=E2+Estep
           D = DENVR(Mol,arg)

           CALL NTERP2(E2,IE,X)                    ! NTERP2 determines IE and X for a k(E)
                                                   ! array with Isize+imax1 elements; of which
                                                   ! imax1 elements are below the reaction threshold
           Y1 = TRate(Mol,nchann,IE-1)
           Y2 = TRate(Mol,nchann,IE)
           Y3 = TRate(Mol,nchann,IE+1)
           CALL QUDINT(Y1,Y2,Y3,X,Y)                  ! Y = interpolated ln k(E)
           RR = D*exp(Y-E2*hor*TT)                    ! k(E) stored as log
           RK = RK + 0.5*Estep*(RR + Rlast)
           Rlast = RR
           QQts=D*exp(-E2*hor*TT)              ! partition function evaluated to determine
           Qts=Qts+0.5*Estep*(QQts+Qtslast)    ! when the calculation of RK can be stopped
           Qtslast=QQts
           Qcheck=dabs(QQts/Qts)
           if(Qcheck.lt.Qthresh)then
             goto 18
           else
             continue
           end if

           if(E2.gt.Emax2)then
           write(*,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           write(KOUT,*) '*** Emax2 exceeded; extend Emax2 in Densum'
           STOP
           else
             continue
           end if
         end do

 18      continue    
       RK = RK*exp(-Eo(Mol,nchann)*hor*TT)/Q
       IF ( NCENT(Mol,nchann).EQ.1 ) THEN               ! If Centrifugal Correction
          K(mm) = RK*(TSmom(Mol,nchann)/MolMom(Mol))    ! kinf with adiabatic Qrot ratio
       ELSE
          K(mm) = RK                                    ! Adiabatic Qrot ratio = 1.0
       ENDIF
1000  CONTINUE

      Kinf = K(2)
      Et = -(LOG(K(3)/K(1)))/(TS(3)-TS(1))                  ! Ea/R
      Ainf = K(2)*exp(Et*TS(2))                             ! 1/s
      Einf = (1.9872065d-03)*Et                             ! kcal/mole
      RETURN
      END
