**==stepper.spg  processed by SPAG 5.11R  at 20:12 on 26 Mar 2001
      SUBROUTINE STEPPER(Mol,E,T,NumbDens,IP,Delt)
 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2002 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c     01/02  v1.2.1  -New Feature: can optionally choose to apply centrifugal
c                     corrections for any choice of model. Thus can apply them
c                     even when using Inverse Laplace Transform form of k(E).
c                     NOTE: this new feature affects the data file format.
c     11/01  v1.1.4  -Bug Fix: previous version applied centrifugal correction 
c                      inconsistently when calculating Arrhenius parameters 
c                     (k(inf) was correct, however).
c     09/01  v1.1.2  -Revised centrigfugal corrections to k(E)'s.
c                     See Robinson & Holbrook (1972) eq. 4.30 and 4.31.
c
c***********************************************************************
c	Carries out Gillespie Exact Stochastic procedure for one step
c
c	Mol	= molecule index
c	E	= energy (relative to ZPE of Mol)
c	T	= translational temperature
c	NumbDens= number density of collider gas
c	IP	= index of selected process
c	Delt	= time step
c***********************************************************************
      IMPLICIT NONE
 
      INCLUDE 'declare1.inc'
      INCLUDE 'declare2.inc'
 
      INTEGER Mol , i , IE , IP
 
      DOUBLE PRECISION Atot , RX , arg , E , X , Delt , Atemp , RAN1 , 
     &                 Y1 , Y2 , Y3 , Y , T , numbdens , Aref , CC ,
     &                 DOUBNTERP
 
      DOUBLE PRECISION vivr , civr , rivr , pivr, tivr, arg2 , RVR

      INTEGER iivr

      COMMON/IVR/ vivr( MaxWell , MaxChan ) , 
     &            pivr( MaxWell , MaxChan ) ,
     &            tivr( MaxWell , MaxChan ) ,
     &            civr( MaxWell , MaxChan , 3 ) , 
     &            iivr( MaxWell , MaxChan )

      EXTERNAL DOUBNTERP

      SAVE 
 
         i = Nchan(Mol) + 1

         CC = DOUBNTERP(E,Mol,CNORM)
         CC = CC/CNORMref(Mol)                   ! NEW up-transition probability collision treatment <<<<<<<<<<

         Rab(i) = CollK(Mol)*numbdens*CC                      ! Inelastic collision frequency: NEW up-transition probability
         Atot = Rab(Nchan(Mol)+1)                             ! Collision frequency

      IF ( Nchan(Mol).GT.0 ) THEN                             ! If reaction channels
         DO 50 i = 1 , Nchan(Mol)
            Rab(i) = 0.0D+00
            arg = E - Eor(Mol,i)                              ! Active energy in TS to determine k(E); centrifugal correction
            arg2 =-imax1*Egrain1                              ! minimum active energy in TS for the tunneling case
               if((itun(Mol,i).eq.1).and.(arg.ge.arg2))then        ! tunneling condition
                 call NTERP2(arg,IE,X)
                 Y1 = log(rivr(Mol,i,IE-1,numbdens))+TRate(Mol,i,IE-1)  ! log (k(E) * IVR transmission coefficient)
                 Y2 = log(rivr(Mol,i,IE,numbdens))+TRate(Mol,i,IE)      ! log (k(E) * IVR transmission coefficient)
                 Y3 = log(rivr(Mol,i,IE+1,numbdens))+TRate(Mol,i,IE+1)  ! log (k(E) * IVR transmission coefficient)
                 CALL QUDINT(Y1,Y2,Y3,X,Y) 
                 Rab(i) = EXP(Y)           ! reminder: k(E) stored as log [now includes IVR effects]
                 Atot = Atot + Rab(i)
               elseif ( arg .GE. 0.0D+00 ) THEN                    ! no tunneling
                 Rab(i) = RVR( Mol , i , arg , IE )                ! k(E) interpolation
     &                      * rivr( Mol , i , IE , numbdens )      ! IVR transmission coefficient
                 Atot = Atot + Rab(i)
               end if
 50      CONTINUE
      ENDIF

      IF ( Atot .GT. 0.0 ) THEN
        RX = RAN1(IDUM)
        Aref = RX*Atot
        Atemp = 0.0D+00
        IP = Nchan(Mol) + 2
        DO WHILE ( (Atemp.LE.Aref) .AND. (IP .GT. 1 ) )
         IP = IP - 1
         Atemp = Atemp + Rab(IP)
        ENDDO         
c       arg = E - Eor(1,1)
c       IF(arg .lt. 10. ) write(*,*) arg , Rab(1) , IP 
        RX = RAN1(IDUM)
        Delt = -LOG(RX)/Atot              ! Determine time step
        IF ( RX .LE. 1.00D-11 ) THEN        ! if random number too small, call another one
          RX = RAN1(IDUM)
          Delt = -LOG(RX)/Atot           ! Determine time step
        ENDIF
      ELSE
        IP =  Nchan(Mol) + 1
        Delt = Tlim
      ENDIF

      RETURN
      END
